# <copyright>
# INTEL CONFIDENTIAL
#
# Copyright 2022 Intel Corporation
#
# This software and the related documents are Intel copyrighted materials, and your use of
# them is governed by the express license under which they were provided to you ("License").
# Unless the License provides otherwise, you may not use, modify, copy, publish, distribute,
# disclose or transmit this software or the related documents without Intel's prior written
# permission.
#
# This software and the related documents are provided as is, with no express or implied
# warranties, other than those that are expressly stated in the License.
#
# <copyright>

[Diagnostics.CodeAnalysis.SuppressMessageAttribute("PSAvoidUsingPositionalParameters", "", Scope="function")] param()

#.ExternalHelp IntelEthernetCmdlets.dll-Help.xml
function Write-IntelEthernetDebugDump
{
    [CmdletBinding(DefaultParameterSetName = 'ByAdapter')]
    Param(
        [parameter(Mandatory=$true, ParameterSetName="ByName")]
        [ValidateNotNullOrEmpty()]
        [String]
        $Name,

        [parameter(Mandatory=$true, Position=0, ValueFromPipeline=$true, ParameterSetName="ByAdapter")]
        [ValidateNotNullOrEmpty()]
        [Object]
        $Adapter,

        [parameter(Mandatory=$true, ParameterSetName="ByName")]
        [parameter(Mandatory=$true,  Position=1, ParameterSetName="ByAdapter")]
        [SupportsWildcards()]
        [ValidateNotNullOrEmpty()]
        [String[]]
        $Cluster,

        [string]
        [parameter(Mandatory=$false, ParameterSetName="ByName")]
        [parameter(Mandatory=$false, Position=2, ParameterSetName="ByAdapter")]
        [ValidateNotNullOrEmpty()]
        $Path = '',

        [parameter(Mandatory=$false, ParameterSetName="ByName")]
        [parameter(Mandatory=$false, ParameterSetName="ByAdapter")]
        [switch]
        $Force = $false,

        [parameter(Mandatory=$false, ParameterSetName="ByName")]
        [parameter(Mandatory=$false, ParameterSetName="ByAdapter")]
        [switch]
        $Append = $false
    )
    Begin
    {
        $script:ErrorMessagesWrite = @()
        $script:WarningMessagesWrite = @()

        GetIntelEthernetDevices
        $script:SupportedAdapters = $script:SupportedAdapters.Where({$_.Service -in @('icea')})

        $Clusters = $Cluster
    }
    End
    {
        do
        {
            if ($Name -match '\*')
            {
                $script:ErrorMessagesWrite += $Messages.AdapterNotFound -f $Name
                break
            }

            $AdapterName = ValidateSetAdapterNameParams $Name $Adapter ([ref]$script:ErrorMessagesWrite)

            if (-not (ValidateSingleAdapter $input $AdapterName ([ref]$script:ErrorMessagesWrite)))
            {
                break
            }

            $AdapterName = GetSupportedAdapters $AdapterName ([ref]$script:ErrorMessagesWrite)

            if (([string]::IsNullOrEmpty($AdapterName)))
            {
                break
            }

            if (([string]::IsNullOrEmpty($Path)))
            {
                $Timestamp = Get-Date -Format "MMddyyyy"
                $DefaultName = ($AdapterName -replace '\s+', '_') + "_$Timestamp`_dump.bin"
                $UseDefaultPath = $true
            }

            $LogPath = ''
            if (-not (ValidatePathParams ([ref]$LogPath) $UseDefaultPath $DefaultName ([ref]$script:ErrorMessagesWrite)))
            {
                break
            }

            $ClusterIDs = ValidateClusterParam $Clusters $AdapterName
            if ($ClusterIDs.Count)
            {
                $Output = [System.Collections.ArrayList]::new()

                # First Dump the 24 Byte Metadata Cluster
                $Metadata = CreateMetadataCluster $AdapterName
                if ($null -eq $Metadata)
                {
                    foreach ($ClusterID in $ClusterIDs)
                    {
                        $script:ErrorMessagesWrite += $Messages.FailedToReadCluster -f $DebugClusterMap.[int]$ClusterID
                    }
                    break
                }

                $Output.AddRange($Metadata)
                $bWriteFile = $false

                foreach ($ClusterID in $ClusterIDs)
                {
                    $ClusterID = [int]$ClusterID
                    $Dump = ReadCluster $ClusterID
                    if ($Dump)
                    {
                        $Output.AddRange($Dump)
                        $bWriteFile = $true
                        $Dump = $null # hint to Garbage Collector that memory can be freed
                    }
                    else
                    {
                        $script:ErrorMessagesWrite += $Messages.FailedToReadCluster -f $DebugClusterMap.[int]$ClusterID
                    }
                }

                if ($bWriteFile)
                {
                    WriteToFile $LogPath $Output $Append
                }
                $Output = $null # hint to Garbage Collector that memory can be freed
            }
            else
            {
                break
            }

        } while ($false)

        foreach ($WarningMessage in $script:WarningMessagesWrite)
        {
            Write-Warning $WarningMessage
        }

        foreach ($ErrorMessage in $script:ErrorMessagesWrite)
        {
            Write-Error $ErrorMessage
        }
        # trigger Garbage Collection
        [System.GC]::Collect()
    }
}


Function CreateMetadataCluster($AdapterName)
{
    $MetadataClusterOutput = [System.Collections.ArrayList]::new()
    $EETrackIDs = @(Get-CimInstance -Namespace "root\wmi" -ClassName IntlLan_EetrackId -Property Id -ErrorAction SilentlyContinue)
    if ($EETrackIDs)
    {
        $EETrackID = ($EETrackIDs.Where({$_.InstanceName -eq $AdapterName})).Id

        if ($EETrackID)
        {
            $MetadataClusterOutput.AddRange([System.BitConverter]::GetBytes(0x80000001))  # Metadata Cluster ID
            $MetadataClusterOutput.AddRange([System.BitConverter]::GetBytes($EETrackID))
            $MetadataClusterOutput.AddRange([byte[]]::new(16)) # Reserved / not used
        }
    }

    $MetadataClusterOutput
}


Function WriteToFile($Path, $Output, $Append)
{
    if ($Append)
    {
        Add-Content $Path -Value $Output -Encoding Byte
    }
    else
    {
        Set-Content $Path -Value $Output -Encoding Byte
    }

    Resolve-Path -Path $Path -ErrorAction SilentlyContinue
}


Function ValidateClusterParam($Clusters, $AdapterName)
{
    $ValidClusterIDs = @()
    $InvalidClusters = @()

    # Throw an error for Invalid Clusters, but still run if there are any valid ones
    foreach ($Cluster in $Clusters)
    {
        $ClusterID = $DebugClusterMap.GetEnumerator().Where({$_.Value -like $Cluster}).Name
        if ($ClusterID.Count -gt 0) # Check for corresponding Keys
        {
            $ValidClusterIDs += $ClusterID
            continue
        }

        # Check if they sent in the numerical Cluster ID
        if($Cluster -match "^\d+$")
        {
            if ($DebugClusterMap.[int]$Cluster) # Check for corresponding Values
            {
                $ValidClusterIDs += $Cluster
                continue
            }
        }

        $InvalidClusters += $Cluster
    }

    $InvalidClusters = $InvalidClusters | Sort-Object | Get-Unique
    foreach ($Cluster in $InvalidClusters)
    {
        $script:ErrorMessagesWrite += $Messages.InvalidCluster -f $AdapterName, $Cluster
    }

    $ValidClusterIDs = $ValidClusterIDs | Sort-Object | Get-Unique
    $ValidClusterIDs
}


Function BuildTableOutput($ClusterID, $TableID, $TableIndex, $TableBufferSize, $TableBuffer)
{
    $Output = [System.Collections.ArrayList]::new()

    $Output.AddRange([System.BitConverter]::GetBytes($ClusterID))
    $Output.AddRange([System.BitConverter]::GetBytes($TableID))
    $Output.AddRange([System.BitConverter]::GetBytes($TableBufferSize))
    $Output.AddRange([System.BitConverter]::GetBytes($TableIndex))
    $Output.AddRange([byte[]]::new(8)) # Reserved Bytes
    $Output.AddRange($TableBuffer)

    $Output
}


Function ReadCluster($ClusterID)
{
    $i = 0
    $j = 0

    $TableID = 0
    $TableIndex = 0
    $ClusterOutput = [System.Collections.ArrayList]::new()

    $FinalTableID = 0xFF
    $FinalTableID_FWBug = 0xFFFF
    $FinalTableIndex = [uint32]0xFFFFFFFFL

    while ($TableID -ne $FinalTableID -and $TableID -ne $FinalTableID_FWBug)
    {
        Write-Progress -Activity $Messages.WritingDebugDump -PercentComplete $j
        # Call the OID for the current buffer and the next iteration info
        Write-Verbose "`[IN`] Buffer $i : $($DebugClusterMap.$ClusterID) Cluster `($ClusterID`)`tTableID $TableID`tTableIndex $TableIndex"
        $Dump = GetDump $ClusterID $TableID $TableIndex
        if ($Dump.TableBufferSize)
        {
            $Output = BuildTableOutput $ClusterID $TableID $TableIndex $Dump.TableBufferSize $Dump.TableBuffer
            $ClusterOutput.AddRange($Output)
        }

        if ($TableID -ne $Dump.NextTableID -or
            $TableIndex -eq $FinalTableIndex)
        {
            $TableIndex = 0
            $TableID = $Dump.NextTableID
        }
        else
        {
            $TableIndex = $Dump.NextTableIndex
        }

        $i++

        if ($j++ -ge 100)
        {
            $j = 0
        }
    }

    $ClusterOutput
}


function GetDump($ClusterID, $TableID, $TableIndex)
{
    $TableBuffer = [System.Collections.ArrayList]::new()
    $TableBufferSize = 0

    $MaxOIDBufs = 8
    for ($BufIndex = 0; $BufIndex -lt $MaxOIDBufs; $BufIndex++)
    {
        # Build Input Buffer: BufIndex, ClusterId, TableId, TableIndex starting at the 4th byte
        $InBuf = [byte[]]::new(1024)
        [System.BitConverter]::GetBytes($BufIndex).CopyTo($InBuf, 4)
        [System.BitConverter]::GetBytes($ClusterID).CopyTo($InBuf, 8)
        [System.BitConverter]::GetBytes($TableID).CopyTo($InBuf, 12)
        [System.BitConverter]::GetBytes($TableIndex).CopyTo($InBuf, 16)

        $params = @{
            Type        = [uint32]1
            Version     = [uint32]1
            Size        = [uint32]1048 # Header size (24 bytes) + Data Size (1024)
            Operation   = [uint32]0x22 # Operation = 0x22 is INTEL_GEN_DATA_OP_GET_DUMP
            OpVersion   = [uint32]1
            DataSize    = [uint32]1024
            InBuf       = $InBuf
        }

        # first call WmiSetGenData
        $Result = InvokeCimMethod "IntlLan_SetGenData" $AdapterName "WmiSetGenData" $Params

        # if the Result is INTEL_GEN_DATA_STATUS_CALL_GET_GEN_DATA - read GenData
        if ($Result.OutStatus -eq 1)
        {
            $Result = Get-CimInstance -Namespace "root\wmi" -ClassName IntlLan_GetGenData -Filter "InstanceName = '$AdapterName'"
            if ($null -eq $Result)
            {
                $script:ErrorMessagesWrite += $Messages.FailedToReadCluster -f $DebugClusterMap.[int]$ClusterID
                break
            }

            # DumpHeader consists of (Little Endian):
            #   UInt32 BufIndex
            #   UInt32 TableId
            #   UInt32 TableIndex
            #   UInt32 TableSize
            #   UInt32 FwError
            #   UInt32 BufSize
            #   Byte DumpBuf[512]

            $OIDHeaderSize = 48
            $DumpParametersHeaderSize = 48

            $GenDataLength = $Result.GenData.Length
            $DumpHeaderLength = $GenDataLength - $OIDHeaderSize
            $DumpHeader = [byte[]]::new($DumpHeaderLength)
            $DumpBufLength = $DumpHeaderLength - $DumpParametersHeaderSize
            $DumpBufContent = [byte[]]::new($DumpBufLength)

            # Strip OID response header
            [System.Array]::ConstrainedCopy($Result.GenData, 24, $DumpHeader, 0, $DumpHeaderLength)

            # Strip DumpParameters header
            [System.Array]::ConstrainedCopy($DumpHeader, 24, $DumpBufContent, 0, $DumpBufLength)

            $BufIndex        = [bitconverter]::ToUInt32($DumpHeader, 0)
            $NextTableId     = [bitconverter]::ToUInt32($DumpHeader, 4)
            $NextTableIndex  = [bitconverter]::ToUInt32($DumpHeader, 8)
            $TableSize       = [bitconverter]::ToUInt32($DumpHeader, 12)
            $FwError         = [bitconverter]::ToUInt32($DumpHeader, 16)
            $BufSize         = [bitconverter]::ToUInt32($DumpHeader, 20)

            if ($BufIndex -eq 0)
            {
                Write-Verbose "`[OUT`] $($DebugClusterMap.$ClusterID) Cluster `($ClusterID`)`tNextTableID $($NextTableID)`tNextTableIndex $($NextTableIndex)"

                $TableID = $NextTableID
                $TableIndex = $NextTableIndex
                $TableBufferSize = $TableSize
            }

            if (-Not $BufSize)
            {
                break
            }

            $TableBuffer.AddRange($DumpBufContent[0..($BufSize-1)])
        }
        else
        {
            $script:ErrorMessagesWrite += $Messages.FailedToReadCluster -f $DebugClusterMap.[int]$ClusterID

            Write-Verbose "`[OUT`] Buffer $($BufIndex) : $($DebugClusterMap.$ClusterID) Cluster `($ClusterID`)`tNextTableID $($NextTableID)`tNextTableIndex $($NextTableIndex)`tFwError: $($FwError)"
            break
        }
    }

    return [PsCustomObject] @{
        NextTableId = $TableID
        NextTableIndex = $TableIndex
        TableBuffer = $TableBuffer
        TableBufferSize = $TableBufferSize
    }
}

# SIG # Begin signature block
# MIIodAYJKoZIhvcNAQcCoIIoZTCCKGECAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCDI1gFhuW4q6Zui
# CR3NUDd1XNz4uMMR/G/Tm+Zu5z8isaCCEfMwggVvMIIEV6ADAgECAhBI/JO0YFWU
# jTanyYqJ1pQWMA0GCSqGSIb3DQEBDAUAMHsxCzAJBgNVBAYTAkdCMRswGQYDVQQI
# DBJHcmVhdGVyIE1hbmNoZXN0ZXIxEDAOBgNVBAcMB1NhbGZvcmQxGjAYBgNVBAoM
# EUNvbW9kbyBDQSBMaW1pdGVkMSEwHwYDVQQDDBhBQUEgQ2VydGlmaWNhdGUgU2Vy
# dmljZXMwHhcNMjEwNTI1MDAwMDAwWhcNMjgxMjMxMjM1OTU5WjBWMQswCQYDVQQG
# EwJHQjEYMBYGA1UEChMPU2VjdGlnbyBMaW1pdGVkMS0wKwYDVQQDEyRTZWN0aWdv
# IFB1YmxpYyBDb2RlIFNpZ25pbmcgUm9vdCBSNDYwggIiMA0GCSqGSIb3DQEBAQUA
# A4ICDwAwggIKAoICAQCN55QSIgQkdC7/FiMCkoq2rjaFrEfUI5ErPtx94jGgUW+s
# hJHjUoq14pbe0IdjJImK/+8Skzt9u7aKvb0Ffyeba2XTpQxpsbxJOZrxbW6q5KCD
# J9qaDStQ6Utbs7hkNqR+Sj2pcaths3OzPAsM79szV+W+NDfjlxtd/R8SPYIDdub7
# P2bSlDFp+m2zNKzBenjcklDyZMeqLQSrw2rq4C+np9xu1+j/2iGrQL+57g2extme
# me/G3h+pDHazJyCh1rr9gOcB0u/rgimVcI3/uxXP/tEPNqIuTzKQdEZrRzUTdwUz
# T2MuuC3hv2WnBGsY2HH6zAjybYmZELGt2z4s5KoYsMYHAXVn3m3pY2MeNn9pib6q
# RT5uWl+PoVvLnTCGMOgDs0DGDQ84zWeoU4j6uDBl+m/H5x2xg3RpPqzEaDux5mcz
# mrYI4IAFSEDu9oJkRqj1c7AGlfJsZZ+/VVscnFcax3hGfHCqlBuCF6yH6bbJDoEc
# QNYWFyn8XJwYK+pF9e+91WdPKF4F7pBMeufG9ND8+s0+MkYTIDaKBOq3qgdGnA2T
# OglmmVhcKaO5DKYwODzQRjY1fJy67sPV+Qp2+n4FG0DKkjXp1XrRtX8ArqmQqsV/
# AZwQsRb8zG4Y3G9i/qZQp7h7uJ0VP/4gDHXIIloTlRmQAOka1cKG8eOO7F/05QID
# AQABo4IBEjCCAQ4wHwYDVR0jBBgwFoAUoBEKIz6W8Qfs4q8p74Klf9AwpLQwHQYD
# VR0OBBYEFDLrkpr/NZZILyhAQnAgNpFcF4XmMA4GA1UdDwEB/wQEAwIBhjAPBgNV
# HRMBAf8EBTADAQH/MBMGA1UdJQQMMAoGCCsGAQUFBwMDMBsGA1UdIAQUMBIwBgYE
# VR0gADAIBgZngQwBBAEwQwYDVR0fBDwwOjA4oDagNIYyaHR0cDovL2NybC5jb21v
# ZG9jYS5jb20vQUFBQ2VydGlmaWNhdGVTZXJ2aWNlcy5jcmwwNAYIKwYBBQUHAQEE
# KDAmMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5jb21vZG9jYS5jb20wDQYJKoZI
# hvcNAQEMBQADggEBABK/oe+LdJqYRLhpRrWrJAoMpIpnuDqBv0WKfVIHqI0fTiGF
# OaNrXi0ghr8QuK55O1PNtPvYRL4G2VxjZ9RAFodEhnIq1jIV9RKDwvnhXRFAZ/ZC
# J3LFI+ICOBpMIOLbAffNRk8monxmwFE2tokCVMf8WPtsAO7+mKYulaEMUykfb9gZ
# pk+e96wJ6l2CxouvgKe9gUhShDHaMuwV5KZMPWw5c9QLhTkg4IUaaOGnSDip0TYl
# d8GNGRbFiExmfS9jzpjoad+sPKhdnckcW67Y8y90z7h+9teDnRGWYpquRRPaf9xH
# +9/DUp/mBlXpnYzyOmJRvOwkDynUWICE5EV7WtgwggYcMIIEBKADAgECAhAz1wio
# kUBTGeKlu9M5ua1uMA0GCSqGSIb3DQEBDAUAMFYxCzAJBgNVBAYTAkdCMRgwFgYD
# VQQKEw9TZWN0aWdvIExpbWl0ZWQxLTArBgNVBAMTJFNlY3RpZ28gUHVibGljIENv
# ZGUgU2lnbmluZyBSb290IFI0NjAeFw0yMTAzMjIwMDAwMDBaFw0zNjAzMjEyMzU5
# NTlaMFcxCzAJBgNVBAYTAkdCMRgwFgYDVQQKEw9TZWN0aWdvIExpbWl0ZWQxLjAs
# BgNVBAMTJVNlY3RpZ28gUHVibGljIENvZGUgU2lnbmluZyBDQSBFViBSMzYwggGi
# MA0GCSqGSIb3DQEBAQUAA4IBjwAwggGKAoIBgQC70f4et0JbePWQp64sg/GNIdMw
# hoV739PN2RZLrIXFuwHP4owoEXIEdiyBxasSekBKxRDogRQ5G19PB/YwMDB/NSXl
# wHM9QAmU6Kj46zkLVdW2DIseJ/jePiLBv+9l7nPuZd0o3bsffZsyf7eZVReqskmo
# PBBqOsMhspmoQ9c7gqgZYbU+alpduLyeE9AKnvVbj2k4aOqlH1vKI+4L7bzQHkND
# brBTjMJzKkQxbr6PuMYC9ruCBBV5DFIg6JgncWHvL+T4AvszWbX0w1Xn3/YIIq62
# 0QlZ7AGfc4m3Q0/V8tm9VlkJ3bcX9sR0gLqHRqwG29sEDdVOuu6MCTQZlRvmcBME
# Jd+PuNeEM4xspgzraLqVT3xE6NRpjSV5wyHxNXf4T7YSVZXQVugYAtXueciGoWnx
# G06UE2oHYvDQa5mll1CeHDOhHu5hiwVoHI717iaQg9b+cYWnmvINFD42tRKtd3V6
# zOdGNmqQU8vGlHHeBzoh+dYyZ+CcblSGoGSgg8sCAwEAAaOCAWMwggFfMB8GA1Ud
# IwQYMBaAFDLrkpr/NZZILyhAQnAgNpFcF4XmMB0GA1UdDgQWBBSBMpJBKyjNRsjE
# osYqORLsSKk/FDAOBgNVHQ8BAf8EBAMCAYYwEgYDVR0TAQH/BAgwBgEB/wIBADAT
# BgNVHSUEDDAKBggrBgEFBQcDAzAaBgNVHSAEEzARMAYGBFUdIAAwBwYFZ4EMAQMw
# SwYDVR0fBEQwQjBAoD6gPIY6aHR0cDovL2NybC5zZWN0aWdvLmNvbS9TZWN0aWdv
# UHVibGljQ29kZVNpZ25pbmdSb290UjQ2LmNybDB7BggrBgEFBQcBAQRvMG0wRgYI
# KwYBBQUHMAKGOmh0dHA6Ly9jcnQuc2VjdGlnby5jb20vU2VjdGlnb1B1YmxpY0Nv
# ZGVTaWduaW5nUm9vdFI0Ni5wN2MwIwYIKwYBBQUHMAGGF2h0dHA6Ly9vY3NwLnNl
# Y3RpZ28uY29tMA0GCSqGSIb3DQEBDAUAA4ICAQBfNqz7+fZyWhS38Asd3tj9lwHS
# /QHumS2G6Pa38Dn/1oFKWqdCSgotFZ3mlP3FaUqy10vxFhJM9r6QZmWLLXTUqwj3
# ahEDCHd8vmnhsNufJIkD1t5cpOCy1rTP4zjVuW3MJ9bOZBHoEHJ20/ng6SyJ6UnT
# s5eWBgrh9grIQZqRXYHYNneYyoBBl6j4kT9jn6rNVFRLgOr1F2bTlHH9nv1HMePp
# GoYd074g0j+xUl+yk72MlQmYco+VAfSYQ6VK+xQmqp02v3Kw/Ny9hA3s7TSoXpUr
# OBZjBXXZ9jEuFWvilLIq0nQ1tZiao/74Ky+2F0snbFrmuXZe2obdq2TWauqDGIgb
# MYL1iLOUJcAhLwhpAuNMu0wqETDrgXkG4UGVKtQg9guT5Hx2DJ0dJmtfhAH2KpnN
# r97H8OQYok6bLyoMZqaSdSa+2UA1E2+upjcaeuitHFFjBypWBmztfhj24+xkc6Zt
# CDaLrw+ZrnVrFyvCTWrDUUZBVumPwo3/E3Gb2u2e05+r5UWmEsUUWlJBl6MGAAjF
# 5hzqJ4I8O9vmRsTvLQA1E802fZ3lqicIBczOwDYOSxlP0GOabb/FKVMxItt1UHeG
# 0PL4au5rBhs+hSMrl8h+eplBDN1Yfw6owxI9OjWb4J0sjBeBVESoeh2YnZZ/WVim
# VGX/UUIL+Efrz/jlvzCCBlwwggTEoAMCAQICEQCmOovGKSxCae+j/hQscnOWMA0G
# CSqGSIb3DQEBCwUAMFcxCzAJBgNVBAYTAkdCMRgwFgYDVQQKEw9TZWN0aWdvIExp
# bWl0ZWQxLjAsBgNVBAMTJVNlY3RpZ28gUHVibGljIENvZGUgU2lnbmluZyBDQSBF
# ViBSMzYwHhcNMjMwMTI1MDAwMDAwWhcNMjQwMTI1MjM1OTU5WjCBuzEQMA4GA1UE
# BRMHMjE4OTA3NDETMBEGCysGAQQBgjc8AgEDEwJVUzEZMBcGCysGAQQBgjc8AgEC
# EwhEZWxhd2FyZTEdMBsGA1UEDxMUUHJpdmF0ZSBPcmdhbml6YXRpb24xCzAJBgNV
# BAYTAlVTMRMwEQYDVQQIDApDYWxpZm9ybmlhMRowGAYDVQQKDBFJbnRlbCBDb3Jw
# b3JhdGlvbjEaMBgGA1UEAwwRSW50ZWwgQ29ycG9yYXRpb24wggGiMA0GCSqGSIb3
# DQEBAQUAA4IBjwAwggGKAoIBgQDGlXgQt1127m+5LYJmaFBzYTdi9cRozQjdZt/+
# +l4e/XT+sHZemhE0o/x+nydO/Ai7lhA6+Rs0Juw3vSXdGjKZLJ2G19nKCeICKfyK
# /rTX7GkdhwZnX8yQJEWU2VuZ/Ad9MtybHiZY3tIlpauWvNmfILMXWL+8DZOzVifw
# KbQgObHaYERAsfT2ZJ0HYp8lo5XepZy8SCeY/kRShw37vMeqNnuJtjnoB81/sZmI
# bZfLL4XxYn3ESyeeXgGhpplv3DkBUFM6YdD7lNXN7yp7h1gCJUA648U9EMOoQAHi
# ewLOPQSX/xgNqHfk3FUbcwZp07/zLrHtKvDNdPFrgkTBz81xGzVHOi48Lh6FBGW7
# UFEq/jI5TOk/AcM5eDvsizy4dA6J0AJ0o3lez+jAP2tDccEt7bn1aaamcjFpDOol
# MktQz0H07XqmRgVT3wHGHiDswDNUCxbk3ervsNG1afAPcFh5Hx4up8hpJoF7YTOf
# 6OHm8k3dRtjF1DC78oNYgqkpuNMCAwEAAaOCAbwwggG4MB8GA1UdIwQYMBaAFIEy
# kkErKM1GyMSixio5EuxIqT8UMB0GA1UdDgQWBBRvq1HVd47vo0fOeEIdOUOyHE5Z
# ojAOBgNVHQ8BAf8EBAMCB4AwDAYDVR0TAQH/BAIwADATBgNVHSUEDDAKBggrBgEF
# BQcDAzBJBgNVHSAEQjBAMDUGDCsGAQQBsjEBAgEGATAlMCMGCCsGAQUFBwIBFhdo
# dHRwczovL3NlY3RpZ28uY29tL0NQUzAHBgVngQwBAzBLBgNVHR8ERDBCMECgPqA8
# hjpodHRwOi8vY3JsLnNlY3RpZ28uY29tL1NlY3RpZ29QdWJsaWNDb2RlU2lnbmlu
# Z0NBRVZSMzYuY3JsMHsGCCsGAQUFBwEBBG8wbTBGBggrBgEFBQcwAoY6aHR0cDov
# L2NydC5zZWN0aWdvLmNvbS9TZWN0aWdvUHVibGljQ29kZVNpZ25pbmdDQUVWUjM2
# LmNydDAjBggrBgEFBQcwAYYXaHR0cDovL29jc3Auc2VjdGlnby5jb20wLgYDVR0R
# BCcwJaAjBggrBgEFBQcIA6AXMBUME1VTLURFTEFXQVJFLTIxODkwNzQwDQYJKoZI
# hvcNAQELBQADggGBAJan0xi9l18PVDDZYZmI6kX64OJRsJvMbeAOd/iCwfoO1W2d
# +vin3je7xjz7TX1tERtLQGbRiraogMGVMco6+BSXJn0mGwpPcxsaRLpNbcbfXPHx
# niq3yO3gbSOrUU1Z9kScqlrqgbaAMKWnwlhy8MYbo9H2wIXCtrZ9kCuwoI9iilKS
# jjQqWs4085hIBzp3Kc32V4fLkuKGH1KYeTRMm8hP4+V0D+vBVFSu7PFh5t8p93y+
# Y/n7icZkyci6GUMUbVi8YK2z5KDr/ohPWS3a+DP+gAtjafTITgucRazljocMCIIg
# eWtL1Gwq0hvcpQOn+pdNGg8vUIXuVtziS9NmJKVfHEnOXtvte0ep9ziC5yaE8ui0
# 3e0ouY6xuhqPY5+mh9Wve5BX5ju1ewEUuImurs0xjtNBr3bscylS3AkssWz0LBko
# gR/aT9luMyUm/rC/2JLMrDlRTuiWergSd1ffFeLGnQBWtvd0snyFK3ZlVbJelDJP
# 47S4Vj1s45eyKo2DYTGCFdcwghXTAgEBMGwwVzELMAkGA1UEBhMCR0IxGDAWBgNV
# BAoTD1NlY3RpZ28gTGltaXRlZDEuMCwGA1UEAxMlU2VjdGlnbyBQdWJsaWMgQ29k
# ZSBTaWduaW5nIENBIEVWIFIzNgIRAKY6i8YpLEJp76P+FCxyc5YwDQYJYIZIAWUD
# BAIBBQCgajAZBgkqhkiG9w0BCQMxDAYKKwYBBAGCNwIBBDAcBgorBgEEAYI3AgEL
# MQ4wDAYKKwYBBAGCNwIBFjAvBgkqhkiG9w0BCQQxIgQgi2/mxYy7BhrpMf6BimdI
# xk2DjqlEo+Knw34SY5aOQk8wDQYJKoZIhvcNAQEBBQAEggGAWibxLDU1h8GaGeh8
# UFt6jd75OZYR0oNnl9NCSx5vzJesIeDooTr1aqUEGe296OBudLAPpjvsCZUJ1nL6
# sINpL8QFjM7GXz35sdKw22ZfYfXtrEQ9b9tIRFv1Yc8r1WtVZi0xC3FNCQAm/aEb
# rnYA2U+CYC5kB8eodCdmd4jT2Oz3aoWYWRem5B0sYY7UGmh7ysj2c7bZX6oVQNRP
# BSPsA3hkl0aAfDtj9ASxJgN/ur8FKNXeziAwKrkdXxYqTmNmwsVyw8m+gV4s4PPk
# FZZampU+nPtLEua3dQTOw6GZFnjtNKZE7EHyzEl2cjtyFU4Kr6bdoQhvaq5yBuby
# 8hLZbxfuraJMgxemZyYYhsIYgbsLXJlnoPq65yuQGqSBh0gGet1o6blq3/liikSm
# m42SsPVBNY8Sb2D4f/xpCOmHQqAYfWzInNOy7QX3VFyXW7IryhqqzJki8VHmDYB9
# Z2NmGVSmqfTwu2Pm0OHkqKexBL9xpvyZ7Ap4UCtUBU2OVzfeoYITUDCCE0wGCisG
# AQQBgjcDAwExghM8MIITOAYJKoZIhvcNAQcCoIITKTCCEyUCAQMxDzANBglghkgB
# ZQMEAgIFADCB7wYLKoZIhvcNAQkQAQSggd8EgdwwgdkCAQEGCisGAQQBsjECAQEw
# MTANBglghkgBZQMEAgEFAAQgOnQDdh3JKwdJNMjcIJ7ukWmZrNb7aIJA2+OaXuGF
# Nt0CFFd/S3omLaKscE2cOoCbjUYDCRCwGA8yMDIzMDUwMjIyMzkzN1qgbqRsMGox
# CzAJBgNVBAYTAkdCMRMwEQYDVQQIEwpNYW5jaGVzdGVyMRgwFgYDVQQKEw9TZWN0
# aWdvIExpbWl0ZWQxLDAqBgNVBAMMI1NlY3RpZ28gUlNBIFRpbWUgU3RhbXBpbmcg
# U2lnbmVyICMzoIIN6jCCBvYwggTeoAMCAQICEQCQOX+a0ko6E/K9kV8IOKlDMA0G
# CSqGSIb3DQEBDAUAMH0xCzAJBgNVBAYTAkdCMRswGQYDVQQIExJHcmVhdGVyIE1h
# bmNoZXN0ZXIxEDAOBgNVBAcTB1NhbGZvcmQxGDAWBgNVBAoTD1NlY3RpZ28gTGlt
# aXRlZDElMCMGA1UEAxMcU2VjdGlnbyBSU0EgVGltZSBTdGFtcGluZyBDQTAeFw0y
# MjA1MTEwMDAwMDBaFw0zMzA4MTAyMzU5NTlaMGoxCzAJBgNVBAYTAkdCMRMwEQYD
# VQQIEwpNYW5jaGVzdGVyMRgwFgYDVQQKEw9TZWN0aWdvIExpbWl0ZWQxLDAqBgNV
# BAMMI1NlY3RpZ28gUlNBIFRpbWUgU3RhbXBpbmcgU2lnbmVyICMzMIICIjANBgkq
# hkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAkLJxP3nh1LmKF8zDl8KQlHLtWjpvAUN/
# c1oonyR8oDVABvqUrwqhg7YT5EsVBl5qiiA0cXu7Ja0/WwqkHy9sfS5hUdCMWTc+
# pl3xHl2AttgfYOPNEmqIH8b+GMuTQ1Z6x84D1gBkKFYisUsZ0vCWyUQfOV2csJbt
# WkmNfnLkQ2t/yaA/bEqt1QBPvQq4g8W9mCwHdgFwRd7D8EJp6v8mzANEHxYo4Wp0
# tpxF+rY6zpTRH72MZar9/MM86A2cOGbV/H0em1mMkVpCV1VQFg1LdHLuoCox/CYC
# NPlkG1n94zrU6LhBKXQBPw3gE3crETz7Pc3Q5+GXW1X3KgNt1c1i2s6cHvzqcH3m
# fUtozlopYdOgXCWzpSdoo1j99S1ryl9kx2soDNqseEHeku8Pxeyr3y1vGlRRbDOz
# jVlg59/oFyKjeUFiz/x785LaruA8Tw9azG7fH7wir7c4EJo0pwv//h1epPPuFjgr
# P6x2lEGdZB36gP0A4f74OtTDXrtpTXKZ5fEyLVH6Ya1N6iaObfypSJg+8kYNabG3
# bvQF20EFxhjAUOT4rf6sY2FHkbxGtUZTbMX04YYnk4Q5bHXgHQx6WYsuy/RkLEJH
# 9FRYhTflx2mn0iWLlr/GreC9sTf3H99Ce6rrHOnrPVrd+NKQ1UmaOh2DGld/HAHC
# zhx9zPuWFcUCAwEAAaOCAYIwggF+MB8GA1UdIwQYMBaAFBqh+GEZIA/DQXdFKI7R
# NV8GEgRVMB0GA1UdDgQWBBQlLmg8a5orJBSpH6LfJjrPFKbx4DAOBgNVHQ8BAf8E
# BAMCBsAwDAYDVR0TAQH/BAIwADAWBgNVHSUBAf8EDDAKBggrBgEFBQcDCDBKBgNV
# HSAEQzBBMDUGDCsGAQQBsjEBAgEDCDAlMCMGCCsGAQUFBwIBFhdodHRwczovL3Nl
# Y3RpZ28uY29tL0NQUzAIBgZngQwBBAIwRAYDVR0fBD0wOzA5oDegNYYzaHR0cDov
# L2NybC5zZWN0aWdvLmNvbS9TZWN0aWdvUlNBVGltZVN0YW1waW5nQ0EuY3JsMHQG
# CCsGAQUFBwEBBGgwZjA/BggrBgEFBQcwAoYzaHR0cDovL2NydC5zZWN0aWdvLmNv
# bS9TZWN0aWdvUlNBVGltZVN0YW1waW5nQ0EuY3J0MCMGCCsGAQUFBzABhhdodHRw
# Oi8vb2NzcC5zZWN0aWdvLmNvbTANBgkqhkiG9w0BAQwFAAOCAgEAc9rtaHLLwrlA
# oTG7tAOjLRR7JOe0WxV9qOn9rdGSDXw9NqBp2fOaMNqsadZ0VyQ/fg882fXDeSVs
# JuiNaJPO8XeJOX+oBAXaNMMU6p8IVKv/xH6WbCvTlOu0bOBFTSyy9zs7WrXB+9eJ
# dW2YcnL29wco89Oy0OsZvhUseO/NRaAA5PgEdrtXxZC+d1SQdJ4LT03EqhOPl68B
# NSvLmxF46fL5iQQ8TuOCEmLrtEQMdUHCDzS4iJ3IIvETatsYL254rcQFtOiECJMH
# +X2D/miYNOR35bHOjJRs2wNtKAVHfpsu8GT726QDMRB8Gvs8GYDRC3C5VV9Hvjlk
# zrfaI1Qy40ayMtjSKYbJFV2Ala8C+7TRLp04fDXgDxztG0dInCJqVYLZ8roIZQPl
# 8SnzSIoJAUymefKithqZlOuXKOG+fRuhfO1WgKb0IjOQ5IRT/Cr6wKeXqOq1jXrO
# 5OBLoTOrC3ag1WkWt45mv1/6H8Sof6ehSBSRDYL8vU2Z7cnmbDb+d0OZuGktfGEv
# 7aOwSf5bvmkkkf+T/FdpkkvZBT9thnLTotDAZNI6QsEaA/vQ7ZohuD+vprJRVNVM
# xcofEo1XxjntXP/snyZ2rWRmZ+iqMODSrbd9sWpBJ24DiqN04IoJgm6/4/a3vJ4L
# KRhogaGcP24WWUsUCQma5q6/YBXdhvUwggbsMIIE1KADAgECAhAwD2+s3WaYdHyp
# RjaneC25MA0GCSqGSIb3DQEBDAUAMIGIMQswCQYDVQQGEwJVUzETMBEGA1UECBMK
# TmV3IEplcnNleTEUMBIGA1UEBxMLSmVyc2V5IENpdHkxHjAcBgNVBAoTFVRoZSBV
# U0VSVFJVU1QgTmV0d29yazEuMCwGA1UEAxMlVVNFUlRydXN0IFJTQSBDZXJ0aWZp
# Y2F0aW9uIEF1dGhvcml0eTAeFw0xOTA1MDIwMDAwMDBaFw0zODAxMTgyMzU5NTla
# MH0xCzAJBgNVBAYTAkdCMRswGQYDVQQIExJHcmVhdGVyIE1hbmNoZXN0ZXIxEDAO
# BgNVBAcTB1NhbGZvcmQxGDAWBgNVBAoTD1NlY3RpZ28gTGltaXRlZDElMCMGA1UE
# AxMcU2VjdGlnbyBSU0EgVGltZSBTdGFtcGluZyBDQTCCAiIwDQYJKoZIhvcNAQEB
# BQADggIPADCCAgoCggIBAMgbAa/ZLH6ImX0BmD8gkL2cgCFUk7nPoD5T77NawHbW
# GgSlzkeDtevEzEk0y/NFZbn5p2QWJgn71TJSeS7JY8ITm7aGPwEFkmZvIavVcRB5
# h/RGKs3EWsnb111JTXJWD9zJ41OYOioe/M5YSdO/8zm7uaQjQqzQFcN/nqJc1zjx
# FrJw06PE37PFcqwuCnf8DZRSt/wflXMkPQEovA8NT7ORAY5unSd1VdEXOzQhe5cB
# lK9/gM/REQpXhMl/VuC9RpyCvpSdv7QgsGB+uE31DT/b0OqFjIpWcdEtlEzIjDzT
# FKKcvSb/01Mgx2Bpm1gKVPQF5/0xrPnIhRfHuCkZpCkvRuPd25Ffnz82Pg4wZytG
# tzWvlr7aTGDMqLufDRTUGMQwmHSCIc9iVrUhcxIe/arKCFiHd6QV6xlV/9A5VC0m
# 7kUaOm/N14Tw1/AoxU9kgwLU++Le8bwCKPRt2ieKBtKWh97oaw7wW33pdmmTIBxK
# lyx3GSuTlZicl57rjsF4VsZEJd8GEpoGLZ8DXv2DolNnyrH6jaFkyYiSWcuoRsDJ
# 8qb/fVfbEnb6ikEk1Bv8cqUUotStQxykSYtBORQDHin6G6UirqXDTYLQjdprt9v3
# GEBXc/Bxo/tKfUU2wfeNgvq5yQ1TgH36tjlYMu9vGFCJ10+dM70atZ2h3pVBeqeD
# AgMBAAGjggFaMIIBVjAfBgNVHSMEGDAWgBRTeb9aqitKz1SA4dibwJ3ysgNmyzAd
# BgNVHQ4EFgQUGqH4YRkgD8NBd0UojtE1XwYSBFUwDgYDVR0PAQH/BAQDAgGGMBIG
# A1UdEwEB/wQIMAYBAf8CAQAwEwYDVR0lBAwwCgYIKwYBBQUHAwgwEQYDVR0gBAow
# CDAGBgRVHSAAMFAGA1UdHwRJMEcwRaBDoEGGP2h0dHA6Ly9jcmwudXNlcnRydXN0
# LmNvbS9VU0VSVHJ1c3RSU0FDZXJ0aWZpY2F0aW9uQXV0aG9yaXR5LmNybDB2Bggr
# BgEFBQcBAQRqMGgwPwYIKwYBBQUHMAKGM2h0dHA6Ly9jcnQudXNlcnRydXN0LmNv
# bS9VU0VSVHJ1c3RSU0FBZGRUcnVzdENBLmNydDAlBggrBgEFBQcwAYYZaHR0cDov
# L29jc3AudXNlcnRydXN0LmNvbTANBgkqhkiG9w0BAQwFAAOCAgEAbVSBpTNdFuG1
# U4GRdd8DejILLSWEEbKw2yp9KgX1vDsn9FqguUlZkClsYcu1UNviffmfAO9Aw63T
# 4uRW+VhBz/FC5RB9/7B0H4/GXAn5M17qoBwmWFzztBEP1dXD4rzVWHi/SHbhRGdt
# j7BDEA+N5Pk4Yr8TAcWFo0zFzLJTMJWk1vSWVgi4zVx/AZa+clJqO0I3fBZ4OZOT
# lJux3LJtQW1nzclvkD1/RXLBGyPWwlWEZuSzxWYG9vPWS16toytCiiGS/qhvWiVw
# YoFzY16gu9jc10rTPa+DBjgSHSSHLeT8AtY+dwS8BDa153fLnC6NIxi5o8JHHfBd
# 1qFzVwVomqfJN2Udvuq82EKDQwWli6YJ/9GhlKZOqj0J9QVst9JkWtgqIsJLnfE5
# XkzeSD2bNJaaCV+O/fexUpHOP4n2HKG1qXUfcb9bQ11lPVCBbqvw0NP8srMftpmW
# JvQ8eYtcZMzN7iea5aDADHKHwW5NWtMe6vBE5jJvHOsXTpTDeGUgOw9Bqh/poUGd
# /rG4oGUqNODeqPk85sEwu8CgYyz8XBYAqNDEf+oRnR4GxqZtMl20OAkrSQeq/eww
# 2vGnL8+3/frQo4TZJ577AWZ3uVYQ4SBuxq6x+ba6yDVdM3aO8XwgDCp3rrWiAoa6
# Ke60WgCxjKvj+QrJVF3UuWp0nr1IrpgxggQtMIIEKQIBATCBkjB9MQswCQYDVQQG
# EwJHQjEbMBkGA1UECBMSR3JlYXRlciBNYW5jaGVzdGVyMRAwDgYDVQQHEwdTYWxm
# b3JkMRgwFgYDVQQKEw9TZWN0aWdvIExpbWl0ZWQxJTAjBgNVBAMTHFNlY3RpZ28g
# UlNBIFRpbWUgU3RhbXBpbmcgQ0ECEQCQOX+a0ko6E/K9kV8IOKlDMA0GCWCGSAFl
# AwQCAgUAoIIBazAaBgkqhkiG9w0BCQMxDQYLKoZIhvcNAQkQAQQwHAYJKoZIhvcN
# AQkFMQ8XDTIzMDUwMjIyMzkzN1owPwYJKoZIhvcNAQkEMTIEMGtMfdoOSVXcaF4/
# 0qboWUGhRV6PbRQJ45MXuMnE0WPld5virpBoAb76479FKkfvwDCB7QYLKoZIhvcN
# AQkQAgwxgd0wgdowgdcwFgQUqzQBOqxAlzGfCBrwsxjhg/gPeIEwgbwEFALWW5Xi
# g3DBVwCV+oj5I92Tf62PMIGjMIGOpIGLMIGIMQswCQYDVQQGEwJVUzETMBEGA1UE
# CBMKTmV3IEplcnNleTEUMBIGA1UEBxMLSmVyc2V5IENpdHkxHjAcBgNVBAoTFVRo
# ZSBVU0VSVFJVU1QgTmV0d29yazEuMCwGA1UEAxMlVVNFUlRydXN0IFJTQSBDZXJ0
# aWZpY2F0aW9uIEF1dGhvcml0eQIQMA9vrN1mmHR8qUY2p3gtuTANBgkqhkiG9w0B
# AQEFAASCAgB45S77Fe8V63O0e2yliFPPwMk1j/B6zrg5+UcW4TAWJcWNDtz0Wrzk
# wbkVMse8JLyA/tUdK2FLE/UnzcfbHPU1z4qJUqjAFZ95o8Jtn2e2C41WFzR/o86c
# /doMoZ9Ak+pelAYhIlQG1SH4atR+ivC3xMX8FUEy4anA3wtACckLgW2rTfMQlvCl
# Nnn97/zSIvPnvhvWHPdiDWHOlyvioHcH3uxg9ig17vFmZC/BQSkx1AqVmP78emtn
# 7GD0HXWd67ob5E4MJ5mPp7onWlE/ZDHlpyKThboxfIENfqUjXCicjFOy7xtzHXSz
# 5Fh+FCGMKlx6zbNhsWDE7KTtJddTw84aB7Gvix0W0b/bZxbhJREQ8aOMZsKoFLzs
# MC3fKtq2zPH1W+Np67bSS4ChYj2hj/3kuDth/V5BIlaw92zbkUZJxmeM82MQWeb/
# sfimuXkLuaXmN0cxbNhNWEFI1aGHatpbq+3X8QDDTjvMHLPRgrd1pYjpG2AiqxrP
# vJ7+ooJwsxKnT8B6y1L+cDQiapdGDMowBjTTmczKqvodw97paACTsJW/g5ahzPfT
# gygZG1bvi4Wo+I0m++OMlV2uyAT6w/DIOg6SD8Unsc0B7ZudRVX0fxLy/5Zt7foe
# 4VcxOOJODU3P9cOHWT1cJ/j/thA9686tdxu/hNfGKt2DikQBnnhH8Q==
# SIG # End signature block
